#include <FastLED.h>
#define NUM_LEDS 60
#define DATA_PIN 5
#define BRIGHTNESS  255
#define LED_TYPE    WS2812B
#define COLOR_ORDER GRB 
#define MUSIC_PIN 7
#define UPDATES_PER_SECOND 10

#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#include <ClickEncoder.h>
#include <TimerOne.h>

CRGB leds[NUM_LEDS];
CRGBPalette16 currentPalette;
TBlendType currentBlending;

#define OLED_RESET 4 // not used with this display
Adafruit_SSD1306 display(OLED_RESET);
ClickEncoder *encoder;
static int16_t last, value;
static uint8_t changed, selected, modus;
static String Paletten[6] = {"Heat", "Rainbow", "Ocean", "Lava", "Forest", "Party"};

void timerIsr() {
  encoder->service();
}

void setup() { 
  currentPalette = HeatColors_p;         
  currentBlending = NOBLEND;
  FastLED.addLeds<LED_TYPE, DATA_PIN, COLOR_ORDER>(leds, NUM_LEDS).setCorrection( TypicalLEDStrip ); 
  FastLED.setBrightness(  BRIGHTNESS ); 
  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);
  encoder = new ClickEncoder(8, 7, 9, 2);
  Timer1.initialize(1000);
  Timer1.attachInterrupt(timerIsr); 
  last = -1; modus = 1; selected = 0;
  for (selected = 0; selected < 6; selected ++) {writeDisplay(Paletten[selected], modus); delay(1000);} 
  selected = 0;
  writeDisplay(Paletten[selected], modus);
}

void loop() {
  //***** first read the music *************************************
  static uint8_t startIndex = 0;
  uint8_t range = 0;
  if (modus == 1) {  
    //***** show the music - display shows "Music"
    int music = analogRead(MUSIC_PIN);
    music = 32 * sqrt(music);
    range = map(music, 0, 1023, 0, NUM_LEDS - 1);
    FillLEDsFromPaletteColors(startIndex, range, modus);  
  }
  else {  
    //***** if not, show the color rolling palette - display shows "static"
    startIndex = startIndex + 1; // motion speed
    FillLEDsFromPaletteColors(startIndex, range, modus);
  }
  FastLED.show();
  //***** now Display and encoder *************************************
  value += encoder->getValue();
  if (value != last) {
    if (value < last) {
      changed = -1;
      selected--;
      if (selected < 0) selected = 5;
    } else {
      changed = 1;
      selected++;
      if (selected > 5) selected = 0;
    }
    last = value;
    if (selected == 0) {currentPalette = HeatColors_p;}
    if (selected == 1) {currentPalette = RainbowColors_p;}
    if (selected == 2) {currentPalette = OceanColors_p;}
    if (selected == 3) {currentPalette = LavaColors_p;}  
    if (selected == 4) {currentPalette = ForestColors_p;}  
    if (selected == 5) {currentPalette = PartyColors_p;} 
    writeDisplay(Paletten[selected], modus);
  }
  else {
   changed = 0; 
  }

  ClickEncoder::Button b = encoder->getButton();
  if (b != ClickEncoder::Open) {
    if (modus == 1) {modus = 2;} else {modus = 1;}
  }
  writeDisplay(Paletten[selected], modus); 
}

void writeDisplay(String Palette, int modus) {
    display.clearDisplay();
    display.setTextColor(WHITE);
    display.setTextSize(2);
    display.setCursor(1, 25);
    display.println(Palette);
    display.setCursor(1, 48);
    if (modus == 1) {display.println("   Music");} else {display.println("  Static");}
    display.display();
}

void FillLEDsFromPaletteColors(uint8_t colorIndex, uint8_t range, uint8_t modus)
{
  if (modus == 1) {
    uint8_t step = 240 / range;
    colorIndex = 0;
    for( int i = range - 1; i > 0; i--) {
      leds[i] = ColorFromPalette( currentPalette, colorIndex, BRIGHTNESS, currentBlending);
      colorIndex += step;
    }
    for (int i = range; i < NUM_LEDS; i++) {
      leds[i] = CRGB::Black;
    }
  }
  if (modus == 2)
  {
    for( int i = 0; i < NUM_LEDS; i++) {
      leds[i] = ColorFromPalette( currentPalette, colorIndex, BRIGHTNESS, currentBlending);
      colorIndex += 2;
    }
  }
}

